//<c>   Copyright 1995-1999 by Gerry J. Danen; all rights reserved.

//<t>   Read FILE_ID.DIZ info from archives

//      Options:
//       /ldc=|     (long desc continuation character)
//      */ldw=###   (long desc width)
//       /ldi=###   (long desc indent)
//      */sub       (process next level of subdirs)


#include"gd_tools.h"
#include"gd_extrn.h"
#include"gd_ansi.h"
#include"gd_msstr.h"
#include<search.h>


#define pgVERS_ "V4.20"
#define pgTITL_ "Create FILES.BBS from FILE_ID.DIZ info from Archive files"
#define pgCOPR_ "1995"


#if     (! LARGE_MODEL)
  #error LARGE MEMORY MODEL REQUIRED
#endif


#define STR_SZ_ 133                             // string size


#define MAXFLS  10000                           // max number of files
#define BUFSIZE 10000
FILE    *fd1, *fd2 ;
UCHAR   fl_1[FS_LEN], fl_2[FS_LEN], work[BUFSIZE+1], tmplocation[FS_LEN] ;
long    found = 0L, cfound = 0L ;
int     iNoInfo=0, ft_entries=0, attrib = REGULAR_FILES ;
struct  find_t dta ;
UCHAR   cmd[256], *pcmd ;
static  char    *ftable[MAXFLS+1] ;
STRING  dizFile = "file_id.diz" ;
char    curdir[FS_LEN] ;
BOOL    ynSubs = FALSE ;
int     nLDI = 0, nLDW = 45 ;
char    cLDC = ' ' ;


union   fileDateUnion
{
        ms_date_t dt ;
        unsigned ndt ;
}       fldate ;


void    do_help( void )
{
    logon(1);
    printf( "\nUsage:\n\t%s [args] ...\n\n"
            "\twhere\t[args] can be any of the following:\n"
            "\t\t /ldc=?    long desc continuation character, default is space\n"
            "\t\t /ldc=pipe long desc continuation is |\n"
            "\t\t*/ldw=###  long desc width, default is 45\n"
            "\t\t /ldi=###  long desc indent, default is 0\n"
            "\t\t*/sub      process next level of subdirs\n\n",
            get_global_program_name() );
    printf( "* means NOT IMPLEMENTED YET\n\n" );
    exit( GD_ERROR_HELP );
}


void    check_for_key( void )
{
    if ( kbhit() )
    {
        if ( getch() == 27 )
        {
            printf( "\nCancelled by user\a\n" );
            exit( GD_ERROR_USER_CANCEL );
        }
    }
}


int     qs_compare( const char **arg1, const char **arg2 )
{
    return( _stricmp( *arg1, *arg2 ) );
}


void    ftable_update( char fname[] )
{
    static char fu_drv[FS_LEN], fu_dir[FS_LEN], fu_name[FS_LEN], fu_ext[FS_LEN] ;

    _splitpath( fname, fu_drv, fu_dir, fu_name, fu_ext );
    if ( ! (
            st_eq(fu_ext,".zip")
            ||
            st_eq(fu_ext,".rar")
            ||
            st_eq(fu_ext,".arj")
       ) )
    {
// printf( "Skipping %s [%s %s]\n", fname, fu_name, fu_ext );
        return ;
    }

    found += 1L ;
    strcpy( cmd, fname );
    st_reduce_spaces( cmd );
    st_trim( cmd );
    strupr( cmd );
    if ( strlen(cmd) )
    {
        if ( ft_entries < MAXFLS )
        {
            ftable[ft_entries] = _strdup(cmd);
            if ( ftable[ft_entries] == NULL )
                exit( GD_ERROR_NO_MEMORY );
            ft_entries += 1 ;
        }
        else
        {
            printf( "Too many entries in the file table: %s skipped\n", fname );
        }
    }
}


void    process_diz( STRING fname )
{
    int         rCount = 0 ;

    if ( ( fd1 = fopen( fname, "r" ) ) == NULL )
    {
        fprintf( fd2, "<<<ERROR>>> cannot open %s\n", fname );
        return ;
    }

    while ( fgets( work, (sizeof(work)-1), fd1 ) )
    {
        check_for_key();
        st_rtrim( work );
        if ( strlen(work) )
        {
            if ( rCount )
            {
                if ( nLDI > 0 )
                    fprintf( fd2, "%*s", nLDI, "" );
                fprintf( fd2, "%c", cLDC );
            }
            fprintf( fd2, "%s\n", work );
            rCount ++ ;
        }
    }
    fclose( fd1 );
}


void    process_entry( char fname[], int iCount, int iTotal )
{
    static int  done ;
    static char fu_drv[FS_LEN], fu_dir[FS_LEN], fu_name[FS_LEN], fu_ext[FS_LEN] ;

    _splitpath( fname, fu_drv, fu_dir, fu_name, fu_ext );

    cfound += 1L ;
    fprintf( stderr, "\r%-40s\r%d of %d: %s", "", iCount, iTotal, fname );
    strupr ( fname );

    fprintf( fd2, "%-13s", fname );
    done = _dos_findfirst( fname, attrib, &dta );
    if ( done )
    {
        fprintf( fd2, "<<<ERROR>>> file is now missing!\n" );
        return ;
    }

    sprintf( work, "%s\\%s", tmplocation, dizFile );
    fl_delete( work );                          // get rid of old file first
    if ( st_eq(fu_ext, ".arj") )
        fl_delete( dizFile );

    check_for_key();

    ST_CLR( work );
    if ( st_eq(fu_ext, ".zip") )
        sprintf( work, "pkunzip %s %s %s >nul\n", fname, dizFile, tmplocation );
    else
    if ( st_eq(fu_ext, ".rar") )
        sprintf( work, "unrar e /c- %s %s %s\\ >nul\n", fname, dizFile, tmplocation );
    else
    if ( st_eq(fu_ext, ".arj") )
        sprintf( work, "arj e -y %s %s -w%s >nul\n", fname, dizFile, tmplocation );
    if ( strlen(work) )
        system( work );
    if ( st_eq(fu_ext, ".arj") )                // arj puts file in current directory
    {
        sprintf( work, "mov %s %s >nul\n", dizFile, tmplocation );
        system( work );
    }

    sprintf( work, "%s\\%s", tmplocation, dizFile );
    if ( ! fl_exist( work ) )
    {
        iNoInfo ++ ;
        fprintf( fd2, "\n" );
        return ;
    }

    process_diz( work );                        // process diz file
}


void    do_parm( UCHAR par[] )
{
    UCHAR       s[5] ;

    ST_CPY( s, par );
    if ( st_eq(s, "sub") )
    {
        ynSubs = TRUE ;
    }
    else
    if ( st_eq(s, "ldc=") )
    {
        if ( st_eq(&par[4], "pipe") )
            cLDC = '|' ;
        else
            cLDC = par[4] ;
        if ( cLDC < 32 || cLDC > 126 )
            cLDC = ' ' ;
    }
    else
    if ( st_eq(s, "ldw=") )
    {
        nLDW = atoi( &par[4] );
    }
    else
    if ( st_eq(s, "ldi=") )
    {
        nLDI = atoi( &par[4] );
    }
    else
    if ( st_eq(s, "?") || st_eq(s, "h") || st_eq(s, "help") )
    {
        do_help();
    }
    else
    {
        logon(1);
        fprintf( stderr, "\nParameter \"%s\" is not legal.\a", par );
        do_help();
    }
}


void    main( int argc, char *argv[] )
{
    int         i, done ;
    UCHAR       a[STR_SZ_] ;

    tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
    ansi_cls( 1, stderr );
    logon(1);

    fl_get_pgm_def( tmplocation, "temp", "", YES );
    if ( ! strlen(tmplocation) )
    {
        printf( "\rEnvironment variable or global .ini entry TEMP must be specified.\n" );
        exit( GD_ERROR_CANCEL );
    }

    argc -= 1 ;
    LOOP( i, 1, argc, 1 )                       // set switches first
    {
        st_cpy( a, argv[i], STR_SZ_ );
        if ( a[0] == '/' )
            do_parm( &a[1] );
    }

    _getcwd( curdir, sizeof(curdir) );          // get current directory

    st_cpy( fl_2, "FILES.BBS", FS_LEN );

    if ( ( fd2 = fopen( fl_2, "w" ) ) == NULL )
    {
        printf( "Cannot open output file: %s\n", fl_2 );
        exit( GD_ERROR_CANNOT_OPEN_FOR_WRITE );
    }

    //  count first how many files to process and load table
    done = _dos_findfirst( "*.*", attrib, &dta );
    while ( ! done )
    {
        ftable_update( dta.name );
        done = _dos_findnext( &dta );
    }

    fprintf( stderr, "Directory %s\n", curdir );
    qsort( (char *)ftable, (size_t)ft_entries, (size_t)sizeof(char *), qs_compare );
    for ( done=0 ; done<ft_entries ; done++ )   // now write the output file
        process_entry( ftable[done], done+1, ft_entries );

    _fcloseall();
    fprintf( stderr, "\r%40s\r%ld archive%s processed\n", "", found, (found==1L ? "" : "s") );

    if ( iNoInfo )
    {
        if ( iNoInfo == ft_entries )
            fprintf( stderr, "No archives have info files (%s)\n", dizFile );
        else
            fprintf( stderr, "%d archive%s without info files (%s)\n", iNoInfo, (iNoInfo==1L ? "" : "s"), dizFile );
    }

    exit( GD_ERROR_NONE );
}
